from django.contrib import admin
from .models import EmailTemplate, EmailMessageLog

class TemplatedEmailMessageAdmin(admin.ModelAdmin):
    list_display = ('name','description', 'subject_content')
    list_editable = ('subject_content', )
    ordering = ('name',)

class MessageLogAdmin(admin.ModelAdmin):
    search_fields = ['to_header', 'subject']
    list_display = ('to_header', 'subject', 'sent')
    ordering = ('-sent',)

admin.site.register(EmailTemplate, TemplatedEmailMessageAdmin)
admin.site.register(EmailMessageLog, MessageLogAdmin)



from django.template.base import (
    Context,
    Template,
    TemplateDoesNotExist
)
from django.template.loader import make_origin
from django.template.loaders.app_directories import Loader as AppDirectoriesLoader
from markdown import markdown
import yaml
from BeautifulSoup import BeautifulSoup
from .base import BaseTemplateMixin


class MarkdownTemplate(Template):
    def __init__(self, template_string, origin=None, name=None):
        split_template = template_string.split("---\n")
        if len(split_template) == 3:
            context = yaml.load(split_template[1])
            if isinstance(context, type({})):
                template_string = split_template[2]
                self.frontmatter = {}
                for k, v in context.iteritems():
                    self.frontmatter[k.lower()] = v
            template_string = markdown(template_string)
            self.text_template = ''.join(
                BeautifulSoup(
                    template_string
                ).findAll(
                    text=True
                )
            )
        super(MarkdownTemplate, self).__init__(
            template_string,
            origin=origin,
            name=name
        )


class MarkdownTemplateLoader(AppDirectoriesLoader):

    def load_template(self, template_name, template_dirs=None):
        source, display_name = self.load_template_source(template_name,
                                                         template_dirs)
        origin = make_origin(display_name, self.load_template_source,
                             template_name, template_dirs)
        try:
            template = get_template_from_string(source, origin, template_name)
            return template, None
        except TemplateDoesNotExist:
            # If compiling the template we found raises TemplateDoesNotExist,
            # back off to returning the source and display name for the
            # template we were asked to load. This allows for correct
            # identification (later) of the actual template that does not
            # exist.
            return source, display_name


class MarkdownTemplateMixin(BaseTemplateMixin):
    markdown_template = None

    def get_md_template(self):
        if self.markdown_template is None:
            loader = MarkdownTemplateLoader()
            self.markdown_template, origin = loader.load_template(
                "%s.md" % self.get_template_name()
            )
        return self.markdown_template

    def get_subject_template(self):
        return Template(self.get_md_template().frontmatter['subject'])

    def get_text_template(self):
        return Template(self.get_md_template().text_template)

    def get_html_template(self):
        return self.get_md_template()

    def get_context_object(self):
        if self._context_obj is None:
            template_context = self.get_md_template().frontmatter
            instance_context = self.get_context()
            instance_context.update(template_context)
            self._context_obj = Context(instance_context)
        return self._context_obj


def get_template_from_string(source, origin=None, name=None):
    """
    Returns a compiled Template object for the given template code,
    handling template inheritance recursively.
    """
    return MarkdownTemplate(source, origin, name)



from .base import BaseTemplateMixin
from classy_mail.models import EmailTemplate
from django.template.base import Template


class ModelTemplateMixin(BaseTemplateMixin):
    _template = None

    def _get_template(self):
        if self._template is None:
            self._template = EmailTemplate.objects.get(
                name=self.get_template_name()
            )
        return self._template

    def get_subject_template(self):
        return Template(self._get_template().subject_content)

    def get_text_template(self):
        return Template(self._get_template().text_content)

    def get_html_template(self):
        template_content = self._get_template().html_content
        if template_content != '':
            return Template(template_content)
        return None



from classy_mail.models import EmailMessageLog

class UniqueMailAlreadySent(Exception):
    pass


class LogMessageMixin(object):
    def send(self, log_msg=True):
        self.msg = self.build_msg()
        # Does this need to be a manager method?
        if log_msg:
            log = EmailMessageLog(
                message_text = self.msg.message().as_string(),
                subject      = self.msg.message().get('Subject'),
                from_header  = self.msg.message().get('From'),
                to_header    = self.msg.message().get('To'),
                cc_header    = self.msg.message().get('Cc'),
                bcc_header   = self.msg.message().get('Bcc'),
            )
            self.msg.send()
            log.save()
        else:
            self.msg.send()

class UniqueMessageMixin(LogMessageMixin):
    def send(self, log_msg=True):
        self.msg = self.build_msg()
        if EmailMessageLog.objects.filter(
            subject      = self.msg.message().get('Subject'),
            to_header    = self.msg.message().get('To'),
            from_header  = self.msg.message().get('From'),
        ).count() > 0:
            raise UniqueMailAlreadySent(
            "Mail found in the log from %s to %s with the subject: %s" % (
                self.msg.message().get('From'),
                self.msg.message().get('To'),
                self.msg.message().get('Subject')
                )
            )
        if log_msg:
            log = EmailMessageLog(
                message_text = self.msg.message().as_string(),
                subject      = self.msg.message().get('Subject'),
                from_header  = self.msg.message().get('From'),
                to_header    = self.msg.message().get('To'),
                cc_header    = self.msg.message().get('Cc'),
                bcc_header   = self.msg.message().get('Bcc'),
            )
            self.msg.send()
            log.save()
        else:
            self.msg.send()



from django.core.mail import EmailMultiAlternatives
from django.template.base import Context, TemplateDoesNotExist
from django.template import loader


def resolve_template(template):
    "Accepts a template object, path-to-template or list of paths"
    if isinstance(template, (list, tuple)):
        return loader.select_template(template)
    elif isinstance(template, basestring):
        try:
            return loader.get_template(template)
        except TemplateDoesNotExist:
            return None
    else:
        return template


class BaseTemplateMixin(object):
    _context_obj = None

    def get_template_name(self):
        return self.template_name

    def get_context(self):
        return self.context

    def get_context_object(self):
        if self._context_obj is None:
            self._context_obj = Context(self.get_context())
        return self._context_obj

    def render_subject_template(self):
        return self.get_subject_template().render(
            self.get_context_object()
        ).lstrip().rstrip().replace('\n', '')

    def render_text_template(self):
        return self.get_text_template().render(self.get_context_object())

    def render_html_template(self):
        return self.get_html_template().render(
            self.get_context_object()
        ).replace("\n", "")

    def build_msg(self):
        if self._msg is None:
            try:
                subject = self.subject.lstrip().rstrip().replace('\n', '')
            except AttributeError:
                subject = self.render_subject_template()
            self._msg = EmailMultiAlternatives(
                subject    = subject,
                body       = self.render_text_template(),
                from_email = self.get_from_email(),
                to         = self.get_to(),
                bcc        = self.get_bcc(),
                cc         = self.get_cc(),
                headers    = self.get_headers()
            )
            if self.get_html_template() is not None:
                self._msg.attach_alternative(
                    self.render_html_template(),
                    'text/html'
                )
        return self._msg



from django.contrib.sites.models import Site


class SiteContextMixin(object):
    def get_context(self):
        context = super(SiteContextMixin, self).get_context()
        context.update(dict(site=Site.objects.get_current()))
        return context



from .base import BaseTemplateMixin, resolve_template


class FileTemplateMixin(BaseTemplateMixin):
    def get_subject_template(self):
        template_name = "%s_subject.txt" % self.get_template_name()
        return resolve_template(template_name)

    def get_text_template(self):
        template_name = "%s_body.txt" % self.get_template_name()
        return resolve_template(template_name)

    def get_html_template(self):
        template_name = "%s_body.html" % self.get_template_name()
        return resolve_template(template_name)



from distutils.core import setup

setup(
    name='django-classy-mail',
    version='0.1.6',
    author='Alex Lovell-Troy',
    author_email='alex@lovelltroy.org',
    description='Class-Based Email for Django built with Mixins',
    packages=['classy_mail', 'classy_mail.mixins' ],
    url='https://github.com/alexlovelltroy/django-classy-mail',
    license='LICENSE.txt',
    long_description=open('README.md').read(),
    install_requires=[
        "Django >= 1.5",
        "markdown >= 2.4",
        "BeautifulSoup >= 3.2",
        "PyYAML >= 3.10",
    ],
)
